<?php

namespace API\script\Gateway;

use API\script\Config\Connection;
use BusinessClass\Form;
use BusinessClass\Question;
use PDO;

/**
 * Permet d'accéder, d'écrire ou de modifier les données contenues dans la table Form
 * afin de gérer l'affichage principale du formulaire.
 */
class GatewayForm
{
    /**
     * @var Connection
     */
    private Connection $connection;

    /**
     *
     */
    public function __construct()
    {
        $this->connection = connect();
    }

    /**
     * Permet d'instancier un formulaire s'il n'y a pas déjà un de présent dans la base de donnée
     * afin de rendre le formulaire modulable.
     *
     * @param Form $form Formulaire que l'on veut sauvegarder
     *
     * @return void
     */
    public function insertForm(Form $form): void
    {
        if (empty($this->getForm())) {
            $query = "INSERT INTO Form(title, description) VALUES(:title, :desc)";
            $this->connection->executeQuery($query, array(
                ':title' => array($form->getTitle(), PDO::PARAM_STR),
                ':desc' => array($form->getDescription(), PDO::PARAM_STR)
            ));
        }
    }

    /**
     * Permet de récupérer le formulaire sauvegarder dans la base de donnée.
     *
     * @return array Retourne un array contenant l'ensemble des données du formulaire
     */
    public function getForm(): array
    {
        $query = "SELECT * FROM Form";
        $this->connection->executeQuery($query);

        return $this->connection->getResults();
    }

    /**
     * Permet de supprimer le formulaire dont l'id est le même que le formulaire passé en paramètre.
     *
     * @param Form $form Formulaire que l'on veut supprimer de la base de donnée
     *
     * @return void
     */
    public function deleteForm(Form $form): void
    {
        $query = "DELETE FROM Form WHERE id = :id";
        $this->connection->executeQuery($query, array(
            ':id' => array($form->getId(), PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de faire le lien entre une catégorie et la réponse possible passées en paramètre
     * à la question dont l'id est aussi passé en paramètre.
     *
     * @param string $keyword Keyword que l'on veut associer
     * @param string $response Réponse que l'on veut associer
     * @param int $idQuestion Id de la question qui contient la réponse
     *
     * @return void
     */
    public function assignKeywordToQuestion(string $keyword, string $response, int $idQuestion): void
    {
        $query = "SELECT pr.id
                  FROM Propose p, PossibleResponse pr
                  WHERE p.question = :id AND p.possibleResponse = pr.id
                        AND pr.content = :response";
        $this->connection->executeQuery($query, array(
            ':id' => array($idQuestion, PDO::PARAM_INT),
            ':response' => array($response, PDO::PARAM_STR)
        ));

        $idPossibleResponse = $this->connection->getResults()[0][0];

        $query = "INSERT INTO Reference(possibleResponse, keyword) VALUES(:possibleResponse, :keyword)";
        $this->connection->executeQuery($query, array(
            ':possibleResponse' => array($idPossibleResponse, PDO::PARAM_INT),
            ':keyword' => array($keyword, PDO::PARAM_STR)
        ));
    }

    /**
     * Permet de supprimer le lien entre une catégorie et la réponse possible passées en paramètre
     * à la question dont l'id est aussi passé en paramètre.
     *
     * @param string $keyword Keyword que l'on veut associer
     * @param string $response Réponse que l'on veut associer
     * @param Question $question Id de la question qui contient la réponse
     *
     * @return void
     */
    public function deleteKeywordFromQuestion(string $keyword, string $response, Question $question): void
    {
        $query = "SELECT pr.id FROM Propose p, PossibleResponse r
                  WHERE p.question = :id AND p.possibleResponse = pr.id
                        AND pr.content = :response";
        $this->connection->executeQuery($query, array(
            ':id' => array($question->getId(), PDO::PARAM_INT),
            ':response' => array($response, PDO::PARAM_STR)
        ));

        $idPossibleResponse = $this->connection->getResults()[0][0];

        $query = "DELETE FROM Reference WHERE response = :idResponse AND keyword = :idKeword";
        $this->connection->executeQuery($query, array(
            ':idResponse' => array($idPossibleResponse, PDO::PARAM_INT),
            ':idKeword' => array($keyword, PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de modifier le titre du formulaire sauvé dans la base de donnée.
     *
     * @param string $title Nouveau titre
     * @param Form $form Formulaire que l'on veut modifier
     *
     * @return void
     */
    public function updateTitleToForm(string $title, Form $form): void
    {
        $query = "UPDATE Form SET title = :title WHERE id = :id";
        $this->connection->executeQuery($query, array(
            ':title' => array($title, PDO::PARAM_STR),
            ':id' => array($form->getId(), PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de modifier la description du formulaire sauvé dans la base de donnée.
     *
     * @param string $description Nouvelle description
     * @param Form $form Formulaire que l'on veut modifier
     *
     * @return void
     */
    public function updateDescriptionToForm(string $description, Form $form): void
    {
        $query = "UPDATE Form SET title = :title WHERE description = :description";
        $this->connection->executeQuery($query, array(
            ':description' => array($description, PDO::PARAM_STR),
            ':id' => array($form->getId(), PDO::PARAM_INT)
        ));
    }

    /**
     * Permet de supprimer la description du formulaire dans la base de donnée en le remplaçant par une chaine vide.
     *
     * @param Form $form Formulaire que l'on veut modifier
     *
     * @return void
     */
    public function deleteDescriptionToForm(Form $form): void
    {
        $query = "UPDATE Form SET title = :title WHERE description = :description";
        $this->connection->executeQuery($query, array(
            ':description' => array('', PDO::PARAM_STR),
            ':id' => array($form->getId(), PDO::PARAM_INT)
        ));
    }
}
